==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/ronportal/controls/comboboxobdobi.php <==
<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;

class ComboBoxObdobi extends ComboBox
{
    const SETTINGS_MAXIMALNI_POCET_OBDOBI = 'maximalnipocetobdobi';

    var $pocatekObdobi;
    var $konecObdobi;
    var $popisObdobi;
    var $uzavreno;


    function __construct($varName = 'obdobi')
    {
        parent::__construct($varName);
    }


//    /**
//     * @return void
//     * @deprecated
//     */
//    function nastavSession()
//    {
//        RonPortalSession::setObdobi($this->pocatekObdobi, $this->konecObdobi);
//    }


    /**
     * @param Database $database
     * @param string $sql
     * @param bool $emptyRow
     * @param int $selectedValue
     * @param bool $translate   Urcuje, jestli se maji prekladat nazvy obdobi
     * @return bool
     */
    function loadSQLObdobi($database, $sql, $emptyRow, $selectedValue, $translate)
    {
        $this->data = [];

        $this->selectedOption = null;

        if ($emptyRow) {
            $this->data[] = new Option('', '');
        }

        $cursor = $database ->getSqlResult($sql, null, 'Chyba pri nacitani dat v ComboBoxObdobi::loadSQLObdobi!');

        $first = true;
        $pocet = 0;
        $jeVybrane = false;

        // prida jednotlive radky
        while ( ($row = $cursor ->fetch())  &&
            (RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI) <= 0  ||  $pocet < RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI)) ) {
            $pocatek = $database->stringToTimestamp($row[1]);
            $konec = $database->stringToTimestamp($row[2]);

            $jeAktualni = (time() >= $pocatek) && (time() < DateUtils::incDay($konec, 1));

            $nastavit = false;
            if ($selectedValue == $pocatek) {
                $nastavit = true;
                $jeVybrane = true;
            } else if ($first || ($jeAktualni && !$jeVybrane)) {
                $nastavit = true;
            }

            $popis = $row[0];

            if ($translate) {
                $popisT = TextManager::_('db_obdobi_popis_' . date('n', $pocatek), '');
                if ($popisT) {
                    $popis = sprintf($popisT, date('Y', $pocatek));
                }
            }

            $this->data[] = $option = new Option( DateUtils::dateToUrlString($pocatek), $popis);

            if ($nastavit) {
                $this->pocatekObdobi = $pocatek;
                $this->konecObdobi = $konec;
                $this->popisObdobi = $popis; // $row[0];
                $this->uzavreno = $row[3];

                $this->selectedOption = $option;
            }

            $first = false;

            if ($pocatek <= time()) { // do maximalniho poctu zapocita jen predchozi obdobi
                $pocet++;
            }
        }
        $cursor ->close();

        if ($first) {
            // V databazi neni zadne obdobi - to je mozne pri pouziti pristupaku bez dalsich aplikaci
            // ! ! ! Co kdyz je v databazi napr. nejake starsi testovaci obdobi jine aplikace ? ? ?
            //       Pak v pristupaku neni mozne vybrat aktualni obdobi
            $maxPocet = RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI);
            if ($maxPocet < 1) {
                $maxPocet = 5;
            }

            $date = DateUtils::firstDayOfMonth(time());
            for ($i = 0; $i < $maxPocet; $i++) {
                $popis = date('n/Y', $date);
                $pocatek = $date;

                $this->data[] = $option = new Option($pocatek, $popis);

                if ($first || ($selectedValue == $pocatek)) {
                    $this->pocatekObdobi = $pocatek;
                    $this->konecObdobi = DateUtils::lastDayOfMonth($date);
                    $this->popisObdobi = $popis;
                    $this->uzavreno = 0;

                    $this ->selectedOption = $option;
                }

                $first = false;

                $date = DateUtils::firstDayOfMonth(DateUtils::incDay($date, -10)); // prejde na predchozi mesic
            }
        }

        return true;
    }


    function nastavSessionVariantni()
    {
        RonPortalSession::setVariantniObdobi($this ->pocatekObdobi, $this->konecObdobi);
    }


    /**
     * Nacte seznam obdobi
     * @param Database $database
     * @param string $appId
     * @param bool $variantni
     * @param bool $pouzeNeuzavrene
     * @param bool $emptyRow
     * @param int  $selectedValue
     * @throws \Exception
     */
    function loadObdobi($database, $appId, $variantni, $pouzeNeuzavrene, $emptyRow, $selectedValue)
    {
        $params = $database ->createParameters();
        if ($variantni) {
            $sql =
                'SELECT PopisObdobi, PocatekObdobi, KonecObdobi, 0 FROM VariantniObdobi' .
                ' ORDER BY PocatekObdobi DESC';
        } else {
            $sql =
                'SELECT DISTINCT Popis, PocatekObdobi, KonecObdobi, Uzavreno FROM Obdobi' .
                ($appId
                    ? ' WHERE Aplikace = ' . $params->addParameter($appId)
                    : " WHERE (Aplikace IN ('ADS', 'APS', 'JIDELNA', 'MZDY'))"
                        // vybere obdobi ze vsech aplikaci (krome skladu)
                        // pozor - muze zpusobit, ze se obdobi opakuje!
                ) .
                ($pouzeNeuzavrene
                    ? ' AND (Uzavreno = 0  OR  Uzavreno IS NULL)'
                    : ''
                ) .
                ' ORDER BY PocatekObdobi DESC';
        }

        $this->loadSQLObdobi($database, new SqlStatement($sql, $params), $emptyRow, $selectedValue, !$variantni);
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2305_171.zip: ronportal/ronportal/controls/comboboxobdobi.php <==
<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;

class ComboBoxObdobi extends ComboBox
{
    const SETTINGS_MAXIMALNI_POCET_OBDOBI = 'maximalnipocetobdobi';

    /** @var ?int       Pocatek vybraneho obdobi */
    public $pocatekObdobi;
    /** @var ?int       Konec vybraneho obdobi */
    public $konecObdobi;
    /** @var ?string    Popis vybraneho obdobi*/
    public $popisObdobi;
    /** @var int        Priznak, jestli je zvolene obdobi uzavreno */
    public $uzavreno;

    public $maximalniPocetObdobi;

    /** @var string     ORDER BY cast SQL dotazu pro nacteni seznamu obdobi */
    public $orderBy = 'PocatekObdobi DESC';

    /** @var null       SQL filtr pro nacteni seznamu obdobi */
    public $sqlFilter = null;

    function __construct($varName = 'obdobi')
    {
        parent::__construct($varName);

        $this ->maximalniPocetObdobi = RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI);
    }


    /**
     * @param string $orderBy
     * @return ComboBoxObdobi
     */
    public function setOrderBy(string $orderBy): ComboBoxObdobi
    {
        $this->orderBy = $orderBy;
        return $this;
    }

    /**
     * @param null $sqlFilter
     * @return ComboBoxObdobi
     */
    public function setSqlFilter($sqlFilter)
    {
        $this->sqlFilter = $sqlFilter;
        return $this;
    }



//    /**
//     * @return void
//     * @deprecated
//     */
//    function nastavSession()
//    {
//        RonPortalSession::setObdobi($this->pocatekObdobi, $this->konecObdobi);
//    }


    /**
     * @param Database $database
     * @param string $sql
     * @param bool $emptyRow
     * @param int $selectedValue
     * @param bool $translate   Urcuje, jestli se maji prekladat nazvy obdobi
     * @return bool
     */
    function loadSQLObdobi($database, $sql, $emptyRow, $selectedValue, $translate)
    {
        $this->data = [];

        $this->selectedOption = null;

        if ($emptyRow) {
            $this->data[] = new Option('', '');
        }

        $cursor = $database ->getSqlResult($sql, null, 'Chyba pri nacitani dat v ComboBoxObdobi::loadSQLObdobi!');

        $first = true;
        $pocet = 0;
        $jeVybrane = false;

        // prida jednotlive radky
        while ( ($row = $cursor ->fetch())  &&
            ($this ->maximalniPocetObdobi <= 0  ||  $pocet < $this ->maximalniPocetObdobi) ) {
            $pocatek = $database->stringToTimestamp($row[1]);
            $konec = $database->stringToTimestamp($row[2]);

            $jeAktualni = (time() >= $pocatek) && (time() < DateUtils::incDay($konec, 1));

            $nastavit = false;
            if ($selectedValue == $pocatek) {
                $nastavit = true;
                $jeVybrane = true;
            } else if ($first || ($jeAktualni && !$jeVybrane)) {
                $nastavit = true;
            }

            $popis = $row[0];

            if ($translate) {
                $popisT = TextManager::_('db_obdobi_popis_' . date('n', $pocatek), '');
                if ($popisT) {
                    $popis = sprintf($popisT, date('Y', $pocatek));
                }
            }

            $this->data[] = $option = new Option( DateUtils::dateToUrlString($pocatek), $popis);

            if ($nastavit) {
                $this->pocatekObdobi = $pocatek;
                $this->konecObdobi = $konec;
                $this->popisObdobi = $popis; // $row[0];
                $this->uzavreno = (int)$row[3];

                $this->selectedOption = $option;
            }

            $first = false;

            if ($pocatek <= time()) { // do maximalniho poctu zapocita jen predchozi obdobi
                $pocet++;
            }
        }
        $cursor ->close();

        if ($first) {
            // V databazi neni zadne obdobi - to je mozne pri pouziti pristupaku bez dalsich aplikaci
            // ! ! ! Co kdyz je v databazi napr. nejake starsi testovaci obdobi jine aplikace ? ? ?
            //       Pak v pristupaku neni mozne vybrat aktualni obdobi
            $maxPocet = RonPortalSettings::getIntValue(self::SETTINGS_MAXIMALNI_POCET_OBDOBI);
            if ($maxPocet < 1) {
                $maxPocet = 5;
            }

            $date = DateUtils::firstDayOfMonth(time());
            for ($i = 0; $i < $maxPocet; $i++) {
                $popis = date('n/Y', $date);
                $pocatek = $date;

                $this->data[] = $option = new Option($pocatek, $popis);

                if ($first || ($selectedValue == $pocatek)) {
                    $this->pocatekObdobi = $pocatek;
                    $this->konecObdobi = DateUtils::lastDayOfMonth($date);
                    $this->popisObdobi = $popis;
                    $this->uzavreno = 0;

                    $this ->selectedOption = $option;
                }

                $first = false;

                $date = DateUtils::firstDayOfMonth(DateUtils::incDay($date, -10)); // prejde na predchozi mesic
            }
        }

        return true;
    }


    function nastavSessionVariantni()
    {
        RonPortalSession::setVariantniObdobi($this ->pocatekObdobi, $this->konecObdobi);
    }


    /**
     * Nacte seznam obdobi
     * @param Database $database
     * @param string $appId
     * @param bool $variantni
     * @param bool $pouzeNeuzavrene
     * @param bool $emptyRow
     * @param int  $selectedValue
     * @throws \Exception
     */
    function loadObdobi($database, $appId, $variantni, $pouzeNeuzavrene, $emptyRow, $selectedValue)
    {
        $params = $database ->createParameters();
        if ($variantni) {
            $sql =
                'SELECT PopisObdobi, PocatekObdobi, KonecObdobi, 0 FROM VariantniObdobi' .
                ($this ->sqlFilter ? ' WHERE ' . $this ->sqlFilter : '') .
                ' ORDER BY ' . $this ->orderBy;
        } else {
            $sql =
                'SELECT DISTINCT Popis, PocatekObdobi, KonecObdobi, Uzavreno FROM Obdobi' .
                ($appId
                    ? ' WHERE Aplikace = ' . $params->addParameter($appId)
                    : " WHERE (Aplikace IN ('ADS', 'APS', 'JIDELNA', 'MZDY'))"
                        // vybere obdobi ze vsech aplikaci (krome skladu)
                        // pozor - muze zpusobit, ze se obdobi opakuje!
                ) .
                ($pouzeNeuzavrene
                    ? ' AND (Uzavreno = 0  OR  Uzavreno IS NULL)'
                    : ''
                ) .
                ($this ->sqlFilter ? ' AND (' . $this ->sqlFilter . ')' : '') .
                ' ORDER BY ' . $this ->orderBy;
        }

        $this->loadSQLObdobi($database, new SqlStatement($sql, $params), $emptyRow, $selectedValue, !$variantni);
    }
}
